classdef PlotFigures_AppDesigner_code < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        Dialog               matlab.ui.Figure
        CanvasPanel          matlab.ui.container.Panel
        ax_Canvas            matlab.ui.control.UIAxes
        ButtonPanel          matlab.ui.container.Panel
        LESMButton           matlab.ui.control.Button
        LineButton           matlab.ui.control.Button
        CircleButton         matlab.ui.control.Button
        TriangleButton       matlab.ui.control.Button
        SquareButton         matlab.ui.control.Button
        ScaleEditField       matlab.ui.control.NumericEditField
        ScaleEditFieldLabel  matlab.ui.control.Label
        Slider               matlab.ui.control.Slider
        ClearButton          matlab.ui.control.Button
        ColorPanel           matlab.ui.container.Panel
        Lamp                 matlab.ui.control.Lamp
        ListBox              matlab.ui.control.ListBox
        GridCheckBox         matlab.ui.control.CheckBox
        YEditField           matlab.ui.control.NumericEditField
        YEditFieldLabel      matlab.ui.control.Label
        XEditField           matlab.ui.control.NumericEditField
        XEditFieldLabel      matlab.ui.control.Label
    end

    
    properties (Access = private)
        canvas = []; % Handle to an object of the Canvas class
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function DialogStartupFcn(app)
            % Initialize objects to manage canvas behavior
            app.canvas = Canvas_2D(app.ax_Canvas,app.Dialog);
                        
            % Turn off axes tool bars
            app.ax_Canvas.Toolbar.Visible = 'off';
            app.ax_Canvas.Interactions = [];
            
            % Turn grid off
            app.canvas.Grid(app.GridCheckBox.Value);
            
            % Turn ruler on
            app.canvas.Ruler(true);

            % Set scale value according to slider value
            app.ScaleEditField.Value = app.Slider.Value;
        end

        % Value changed function: ListBox
        function ListBoxValueChanged(app, event)
            value = app.ListBox.Value;
            switch value
                case 'Red'
                    app.Lamp.Color = [1 0 0];
                case 'Green'
                    app.Lamp.Color = [0 1 0];
                case 'Blue'
                    app.Lamp.Color = [0 0 1];
                case 'Random'
                    app.Lamp.Color = rand(1,3);
            end

        end

        % Value changed function: GridCheckBox
        function GridCheckBoxValueChanged(app, event)
            value = app.GridCheckBox.Value;
            app.canvas.Grid(value);

        end

        % Button pushed function: ClearButton
        function ClearButtonPushed(app, event)
            % Open dialog box to confirm user action.
            % If selected choice is "Yes", clean all objects drawn in the 
            % canvas object.
            choice = questdlg('Are you sure?!','Clear Canvas','Yes','Cancel','Cancel');
            if strcmp(choice,'Yes')
                app.canvas.Clean();
            end

        end

        % Button pushed function: SquareButton
        function SquareButtonPushed(app, event)
            % Get handle to axes associated to canvas
            where = app.canvas.getContext();
            
            % Get X and Y coordinates
            x = app.XEditField.Value;
            y = app.YEditField.Value;
            
            % Check if input data is valid (isnan -> Boolean for "is not a number?")
            if isnan(x) || isnan(y)
                msgbox('Invalid input data!', 'Error','error');
                return
            end
            
            % Plot a square
            square(where,x,y,app.ScaleEditField.Value,app.Lamp.Color) % Auxiliary function for plotting squares
            
            % Adjust axes limits
            app.canvas.fit2view();

        end

        % Button pushed function: TriangleButton
        function TriangleButtonPushed(app, event)
            % Get handle to axes associated to canvas
            where = app.canvas.getContext();
            
            % Get X and Y coordinates
            x = app.XEditField.Value;
            y = app.YEditField.Value;
            
            % Check if input data is valid (isnan -> Boolean for "is not a number?")
            if isnan(x) || isnan(y)
                msgbox('Invalid input data!', 'Error','error');
                return
            end
            
            % Plot a triangle
            triangle(where,x,y,...
                     app.ScaleEditField.Value,app.ScaleEditField.Value,...
                     -pi/2,app.Lamp.Color) % Auxiliary function for plotting triangles
            
            % Adjust axes limits
            app.canvas.fit2view();

        end

        % Button pushed function: CircleButton
        function CircleButtonPushed(app, event)
            % Get handle to axes associated to canvas
            where = app.canvas.getContext();
            
            % Get X and Y coordinates
            x = app.XEditField.Value;
            y = app.YEditField.Value;
            
            % Check if input data is valid (isnan -> Boolean for "is not a number?")
            if isnan(x) || isnan(y)
                msgbox('Invalid input data!', 'Error','error');
                return
            end
            
            % Plot a circle
            circle(where,x,y,app.ScaleEditField.Value*0.5,app.Lamp.Color) % Auxiliary function for plotting circles
            
            % Adjust axes limits
            app.canvas.fit2view();

        end

        % Button pushed function: LineButton
        function LineButtonPushed(app, event)
            % Get handle to axes associated to canvas
            where = app.canvas.getContext();
            
            % Open a dialog window to fill the initial and final coordinates
            % (the dialog returns a cell array with the input data)
            labels = {'Xi:','Yi:','Xf:','Yf:'}; % label of each input field
            title = 'Line'; % dialog window title
            lines = 1; % number of possible lines for each input field
            defaultans = {'0','0','0','0'}; % default answers when nothing is provided
            coords = inputdlg(labels,title,lines,defaultans); % returned values in a cell array

            % Check if the vector of returned values with the line initial and final
            % coordinates is empty.
            % If so, this function ends and nothing is done ("return"). Otherwise,
            % convert the cell array of answers (chars) to a numeric array.
            if isempty(coords)
                return
            else
                coords = str2double(coords);
                % Check if Xi == Xf and Yi == Yf. If so, there's no line to
                % draw.
                if (coords(1) == coords(3)) && (coords(2) == coords(4))
                    return
                end
            end
            
            % Plot a line
            line(where,coords([1 3]),coords([2 4]),'Color',app.Lamp.Color,'Linewidth',1.5)
            
            % Adjust axes limits
            app.canvas.fit2view();

        end

        % Button pushed function: LESMButton
        function LESMButtonPushed(app, event)
            % Get handle to axes associated to canvas
            where = app.canvas.getContext();
            
            % Get X and Y coordinates
            x = app.XEditField.Value;
            y = app.YEditField.Value;
            
            % Check if input data is valid (isnan -> Boolean for "is not a number?")
            if isnan(x) || isnan(y)
                msgbox('Invalid input data!', 'Error','error');
                return
            end
            
            % Plot 'LESM'
            LESM_letters(where,x,y,app.ScaleEditField.Value,app.Lamp.Color);
            
            % Adjust axes limits
            app.canvas.fit2view();

        end

        % Value changed function: ScaleEditField
        function ScaleEditFieldValueChanged(app, event)
            value = app.ScaleEditField.Value;
            
            % Check if input data is valid (isnan -> Boolean for "is not a number?")
            if isnan(value)
                msgbox('Invalid input data!', 'Error','error');
                return
            end
            
            % Update slider position to the new scale value
            if value > app.Slider.Limits(2)
                app.Slider.Value = app.Slider.Limits(2);
            elseif value < app.Slider.Limits(1)
                app.Slider.Value = app.Slider.Limits(1);
            else
                app.Slider.Value = value;
            end

        end

        % Value changed function: Slider
        function SliderValueChanged(app, event)
            value = app.Slider.Value;
            app.ScaleEditField.Value = value;
            
        end

        % Value changing function: Slider
        function SliderValueChanging(app, event)
            changingValue = event.Value;
             app.ScaleEditField.Value = changingValue;

        end

        % Size changed function: CanvasPanel
        function CanvasPanelSizeChanged(app, event)
            position = app.CanvasPanel.Position;
            panel_width = position(3);
            panel_height = position(4);
            canvas_x = 1;
            canvas_y = 1;
            canvas_width = panel_width-1;
            canvas_height = panel_height-1;
            app.ax_Canvas.Position = [canvas_x canvas_y canvas_width canvas_height];
            
            if ~isempty(app.canvas)
                app.canvas.fit2view();
            end

        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create Dialog and hide until all components are created
            app.Dialog = uifigure('Visible', 'off');
            app.Dialog.Position = [100 100 640 558];
            app.Dialog.Name = 'UI Dialog';
            app.Dialog.Scrollable = 'on';

            % Create XEditFieldLabel
            app.XEditFieldLabel = uilabel(app.Dialog);
            app.XEditFieldLabel.HorizontalAlignment = 'right';
            app.XEditFieldLabel.Position = [27 187 25 22];
            app.XEditFieldLabel.Text = 'X';

            % Create XEditField
            app.XEditField = uieditfield(app.Dialog, 'numeric');
            app.XEditField.RoundFractionalValues = 'on';
            app.XEditField.ValueDisplayFormat = '%.0f';
            app.XEditField.HorizontalAlignment = 'center';
            app.XEditField.Position = [67 187 92 30];

            % Create YEditFieldLabel
            app.YEditFieldLabel = uilabel(app.Dialog);
            app.YEditFieldLabel.HorizontalAlignment = 'right';
            app.YEditFieldLabel.Position = [180 187 25 22];
            app.YEditFieldLabel.Text = 'Y';

            % Create YEditField
            app.YEditField = uieditfield(app.Dialog, 'numeric');
            app.YEditField.RoundFractionalValues = 'on';
            app.YEditField.ValueDisplayFormat = '%.0f';
            app.YEditField.HorizontalAlignment = 'center';
            app.YEditField.Position = [220 187 92 30];

            % Create GridCheckBox
            app.GridCheckBox = uicheckbox(app.Dialog);
            app.GridCheckBox.ValueChangedFcn = createCallbackFcn(app, @GridCheckBoxValueChanged, true);
            app.GridCheckBox.Text = 'Grid';
            app.GridCheckBox.Position = [559 205 50 34];
            app.GridCheckBox.Value = true;

            % Create ColorPanel
            app.ColorPanel = uipanel(app.Dialog);
            app.ColorPanel.TitlePosition = 'centertop';
            app.ColorPanel.Title = 'Color';
            app.ColorPanel.Position = [349 148 162 113];

            % Create ListBox
            app.ListBox = uilistbox(app.ColorPanel);
            app.ListBox.Items = {'Red', 'Green', 'Blue', 'Random'};
            app.ListBox.ValueChangedFcn = createCallbackFcn(app, @ListBoxValueChanged, true);
            app.ListBox.Position = [9 9 100 74];
            app.ListBox.Value = 'Red';

            % Create Lamp
            app.Lamp = uilamp(app.ColorPanel);
            app.Lamp.Position = [117 28 36 36];
            app.Lamp.Color = [1 0 0];

            % Create ClearButton
            app.ClearButton = uibutton(app.Dialog, 'push');
            app.ClearButton.ButtonPushedFcn = createCallbackFcn(app, @ClearButtonPushed, true);
            app.ClearButton.Position = [533 157 76 30];
            app.ClearButton.Text = 'Clear';

            % Create Slider
            app.Slider = uislider(app.Dialog);
            app.Slider.Limits = [0 10];
            app.Slider.MajorTicks = [0 1 2 3 4 5 6 7 8 9 10];
            app.Slider.MajorTickLabels = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10'};
            app.Slider.ValueChangedFcn = createCallbackFcn(app, @SliderValueChanged, true);
            app.Slider.ValueChangingFcn = createCallbackFcn(app, @SliderValueChanging, true);
            app.Slider.Position = [47 47 540 3];
            app.Slider.Value = 5;

            % Create ScaleEditFieldLabel
            app.ScaleEditFieldLabel = uilabel(app.Dialog);
            app.ScaleEditFieldLabel.HorizontalAlignment = 'right';
            app.ScaleEditFieldLabel.Position = [269 63 36 22];
            app.ScaleEditFieldLabel.Text = 'Scale';

            % Create ScaleEditField
            app.ScaleEditField = uieditfield(app.Dialog, 'numeric');
            app.ScaleEditField.ValueDisplayFormat = '%.3f';
            app.ScaleEditField.ValueChangedFcn = createCallbackFcn(app, @ScaleEditFieldValueChanged, true);
            app.ScaleEditField.HorizontalAlignment = 'center';
            app.ScaleEditField.Position = [320 63 44 22];

            % Create ButtonPanel
            app.ButtonPanel = uipanel(app.Dialog);
            app.ButtonPanel.Position = [13 90 596 50];

            % Create SquareButton
            app.SquareButton = uibutton(app.ButtonPanel, 'push');
            app.SquareButton.ButtonPushedFcn = createCallbackFcn(app, @SquareButtonPushed, true);
            app.SquareButton.Position = [12 10 108 30];
            app.SquareButton.Text = 'Square';

            % Create TriangleButton
            app.TriangleButton = uibutton(app.ButtonPanel, 'push');
            app.TriangleButton.ButtonPushedFcn = createCallbackFcn(app, @TriangleButtonPushed, true);
            app.TriangleButton.Position = [129 10 108 30];
            app.TriangleButton.Text = 'Triangle';

            % Create CircleButton
            app.CircleButton = uibutton(app.ButtonPanel, 'push');
            app.CircleButton.ButtonPushedFcn = createCallbackFcn(app, @CircleButtonPushed, true);
            app.CircleButton.Position = [247 10 108 30];
            app.CircleButton.Text = 'Circle';

            % Create LineButton
            app.LineButton = uibutton(app.ButtonPanel, 'push');
            app.LineButton.ButtonPushedFcn = createCallbackFcn(app, @LineButtonPushed, true);
            app.LineButton.Position = [363 10 108 30];
            app.LineButton.Text = 'Line';

            % Create LESMButton
            app.LESMButton = uibutton(app.ButtonPanel, 'push');
            app.LESMButton.ButtonPushedFcn = createCallbackFcn(app, @LESMButtonPushed, true);
            app.LESMButton.Position = [479 10 108 30];
            app.LESMButton.Text = 'LESM';

            % Create CanvasPanel
            app.CanvasPanel = uipanel(app.Dialog);
            app.CanvasPanel.AutoResizeChildren = 'off';
            app.CanvasPanel.SizeChangedFcn = createCallbackFcn(app, @CanvasPanelSizeChanged, true);
            app.CanvasPanel.Position = [1 271 640 288];

            % Create ax_Canvas
            app.ax_Canvas = uiaxes(app.CanvasPanel);
            app.ax_Canvas.Position = [1 2 639 286];

            % Show the figure after all components are created
            app.Dialog.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = PlotFigures_AppDesigner_code

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.Dialog)

            % Execute the startup function
            runStartupFcn(app, @DialogStartupFcn)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.Dialog)
        end
    end
end